*$ CREATE SOURCE.FOR
*COPY SOURCE
*
*=== source ===========================================================*
*
      SUBROUTINE SOURCE ( NOMORE )

      INCLUDE '(DBLPRC)'
      INCLUDE '(DIMPAR)'
      INCLUDE '(IOUNIT)'
*
*----------------------------------------------------------------------*
*                                                                      *
*     Copyright (C) 1990-2009      by    Alfredo Ferrari & Paola Sala  *
*     All Rights Reserved.                                             *
*                                                                      *
*                                                                      *
*     New source for FLUKA9x-FLUKA20xy:                                *
*                                                                      *
*     Created on 07 january 1990   by    Alfredo Ferrari & Paola Sala  *
*                                                   Infn - Milan       *
*                                                                      *
*     Last change on 08-feb-09     by    Alfredo Ferrari               *
*                                                                      *
*  This is just an example of a possible user written source routine.  *
*  note that the beam card still has some meaning - in the scoring the *
*  maximum momentum used in deciding the binning is taken from the     *
*  beam momentum.  Other beam card parameters are obsolete.            *
*                                                                      *
*       Output variables:                                              *
*                                                                      *
*              Nomore = if > 0 the run will be terminated              *
*                                                                      *
*----------------------------------------------------------------------*
*
      INCLUDE '(BEAMCM)'
      INCLUDE '(FHEAVY)'
      INCLUDE '(FLKSTK)'
      INCLUDE '(IOIOCM)'
      INCLUDE '(LTCLCM)'
      INCLUDE '(PAPROP)'
      INCLUDE '(SOURCM)'
      INCLUDE '(SUMCOU)'
*
      LOGICAL LFIRST
*
      SAVE LFIRST
      DATA LFIRST / .TRUE. /
*======================================================================*
*                                                                      *
*                 BASIC VERSION                                        *
*                                                                      *
*======================================================================*
      NOMORE = 0
*  +-------------------------------------------------------------------*
*  |  First call initializations:
      IF ( LFIRST ) THEN
*  |  *** The following 3 cards are mandatory ***
         TKESUM = ZERZER
         LFIRST = .FALSE.
         LUSSRC = .TRUE.
*  |  *** User initialization ***
         WRITE(LUNOUT,*)
         WRITE(LUNOUT,'(A,132A)') ("*",i=1,132)
         WRITE(LUNOUT,*)
         WRITE(LUNOUT,*)
         WRITE(LUNOUT,'(A,A)') " Source: X-ray spectrum from a ",
     &      "laser plasma: C/E exp(-E/T_h)"
         WRITE(LUNOUT,'(A,1p,G12.5)') " Temperature of this run: ",
     &      WHASOU(1)
         WRITE(LUNOUT,*)
         WRITE(LUNOUT,*)
         WRITE(LUNOUT,'(A,132A)') ("*",i=1,132)
         WRITE(LUNOUT,*)
       END IF
*  |
*  +-------------------------------------------------------------------*
*  Push one source particle to the stack. Note that you could as well
*  push many but this way we reserve a maximum amount of space in the
*  stack for the secondaries to be generated
* Npflka is the stack counter: of course any time source is called it
* must be =0
      NPFLKA = NPFLKA + 1
      ran01 = flrndm(tkesum)
*     Emax =200 MeV
      energy = 1.d-5 + ( 2.d-1 - 1.d-5) * ran01
* !because in the formula the unit should be KeV
      enkev = energy*1.d6    
*      WRITE(LUNOUT,'(A,1p,G12.5)') " energy: ",
*     &      enkev
* We set Whasou(1) = T_h (T_h = temperature) 
      weight = (enkev**2/whasou(1)**3)*
     &         exp(-enkev/whasou(1))    
*      WRITE(LUNOUT,'(A,1p,G12.5)') " weight: ",
*     &      weight
* Wt is the weight of the particle
      wtflk  (npflka) = weight
      WEIPRI = WEIPRI + WTFLK (NPFLKA)
* Particle type (1=proton.....). Ijbeam is the type set by the BEAM
* card
*  +-------------------------------------------------------------------*
*  |  (Radioactive) isotope:
      IF ( IJBEAM .EQ. -2 .AND. LRDBEA ) THEN
         IARES  = IPROA
         IZRES  = IPROZ
         IISRES = IPROM
         CALL STISBM ( IARES, IZRES, IISRES )
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
*  |
*  +-------------------------------------------------------------------*
*  |  Heavy ion:
      ELSE IF ( IJBEAM .EQ. -2 ) THEN
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
         ILOFLK (NPFLKA) = IJHION
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*  |  Group number for "low" energy neutrons, set to 0 anyway
         IGROUP (NPFLKA) = 0
*  |
*  +-------------------------------------------------------------------*
*  |  Normal hadron:
      ELSE
         IONID = IJBEAM
         ILOFLK (NPFLKA) = IJBEAM
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*    Group number for "low" energy neutrons, set to 0 anyway
         IGROUP (NPFLKA) = 0
      END IF
*  |
*  +-------------------------------------------------------------------*
* From this point .....
* Particle generation (1 for primaries)
      LOFLK  (NPFLKA) = 1
* User dependent flag:
      LOUSE  (NPFLKA) = 0
* User dependent spare variables:
      DO 100 ISPR = 1, MKBMX1
         SPAREK (ISPR,NPFLKA) = ZERZER
 100  CONTINUE
* User dependent spare flags:
      DO 200 ISPR = 1, MKBMX2
         ISPARK (ISPR,NPFLKA) = 0
 200  CONTINUE
* Save the track number of the stack particle:
      ISPARK (MKBMX2,NPFLKA) = NPFLKA
      NPARMA = NPARMA + 1
      NUMPAR (NPFLKA) = NPARMA
      NEVENT (NPFLKA) = 0
      DFNEAR (NPFLKA) = +ZERZER
* ... to this point: don't change anything
* Particle age (s)
      AGESTK (NPFLKA) = +ZERZER
      AKNSHR (NPFLKA) = -TWOTWO
      
      IGROUP (NPFLKA) = 0
* Kinetic energy of the particle (GeV)
*     TKEFLK (NPFLKA) = SQRT ( PBEAM**2 + AM (IONID)**2 ) - AM (IONID)
      TKEFLK (NPFLKA) = energy
* Particle momentum      
      PMOFLK (NPFLKA) = SQRT ( TKEFLK (NPFLKA) * ( TKEFLK (NPFLKA)
     &                       + TWOTWO * AM (IONID) ) )
* Cosines (tx,ty,tz)
*************************************************************
******** FROM ALBERTO FASSO**********************************
*  Sample the cosine of the polar angle
*      COSTHE = TWOTWO * FLRNDM(energy) - ONEONE
*      SINTHE = SQRT(ONEONE - COSTHE**2)
*      IF(FLRNDM(energy) .LE. HLFHLF) SINTHE = -SINTHE
**  Samples the azimuthal angle
*      PHI = TWOPIP * FLRNDM(COSTHE)
*      COSPHI = COS(PHI)
**  Cosines (tx,ty,tz) (make sure they are properly normalized)
*      TZFLK (NPFLKA) = COSTHE
*      TYFLK (NPFLKA) = SINTHE * COSPHI
*      TXFLK (NPFLKA) = SQRT(ONEONE - TZFLK(NPFLKA)**2
*     &                             - TYFLK(NPFLKA)**2)
*      IF(FLRNDM(energy) .LE. HLFHLF)
*     &                      TXFLK(NPFLKA) = -TXFLK(NPFLKA)
*  commented by Kelly 05/03/10
*      CALL RACO(UUU,VVV,WWW)
*       TZFLK (NPFLKA) = 1.0
*       TYFLK (NPFLKA) = 0.0
*       TXFLK (NPFLKA) = 0.0
* Cosines (tx,ty,tz)
      COSTNN = energy+2*5.11d-4
      THECOS = SQRT(energy/COSTNN)
      COSTHE = THECOS+FLRNDM(X)*(1-THECOS) 
      SINTHE = SQRT(ONEONE - COSTHE**2)
*  Samples the azimuthal angle
      PHI = TWOPIP * FLRNDM(X)
      COSPHI = COS(PHI)
**  Cosines (tx,ty,tz) (make sure they are properly normalized)
      TZFLK (NPFLKA) = COSTHE
      TYFLK (NPFLKA) = SINTHE * COSPHI
      TXFLK (NPFLKA) = SQRT(ONEONE - TZFLK(NPFLKA)**2
     &                             - TYFLK(NPFLKA)**2)
      IF(FLRNDM(X) .LE. HLFHLF)
     &                      TXFLK(NPFLKA) = -TXFLK(NPFLKA)
* Polarization cosines:
      TXPOL  (NPFLKA) = -TWOTWO
      TYPOL  (NPFLKA) = +ZERZER
      TZPOL  (NPFLKA) = +ZERZER
* Particle coordinates
      XFLK   (NPFLKA) = XBEAM
      YFLK   (NPFLKA) = YBEAM
      ZFLK   (NPFLKA) = ZBEAM
*  Calculate the total kinetic energy of the primaries: don't change
      IF ( ILOFLK (NPFLKA) .EQ. -2 .OR. ILOFLK (NPFLKA) .GT. 100000 )
     &   THEN
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      ELSE IF ( ILOFLK (NPFLKA) .NE. 0 ) THEN
         TKESUM = TKESUM + ( TKEFLK (NPFLKA) + AMDISC (ILOFLK(NPFLKA)) )
     &          * WTFLK (NPFLKA)
      ELSE
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      END IF
      RADDLY (NPFLKA) = ZERZER
*  Here we ask for the region number of the hitting point.
*     NREG (NPFLKA) = ...
*  The following line makes the starting region search much more
*  robust if particles are starting very close to a boundary:
      CALL GEOCRS ( TXFLK (NPFLKA), TYFLK (NPFLKA), TZFLK (NPFLKA) )
      CALL GEOREG ( XFLK  (NPFLKA), YFLK  (NPFLKA), ZFLK  (NPFLKA),
     &              NRGFLK(NPFLKA), IDISC )
*  Do not change these cards:
      CALL GEOHSM ( NHSPNT (NPFLKA), 1, -11, MLATTC )
      NLATTC (NPFLKA) = MLATTC
      CMPATH (NPFLKA) = ZERZER
      CALL SOEVSV
      RETURN
*=== End of subroutine Source =========================================*
      END

